#include "override.h"

HDC CBitmapCache::CreateDC()
{
	if(!m_hdc) {
		m_hdc = CreateCompatibleDC(NULL);
	}
	return m_hdc;
}

HBITMAP CBitmapCache::CreateDIB(int width, int height, BYTE** lplpPixels)
{
	SIZE& dibSize = m_dibSize;
	width  = (width + 3) & ~3;

	if (dibSize.cx < width || dibSize.cy < height) {
		if (dibSize.cx > width) {
			width = dibSize.cx;
		}
		if (dibSize.cy > height) {
			height = dibSize.cy;
		}
		TRACE(_T("width=%d, height=%d\n"), width, height);

		BITMAPINFOHEADER bmiHeader = { sizeof(BITMAPINFOHEADER), width, -height, 1, 32, BI_RGB };
		HBITMAP hbmpNew = CreateDIBSection(CreateDC(), (BITMAPINFO*)&bmiHeader, DIB_RGB_COLORS, (LPVOID*)lplpPixels, NULL, 0);
		if (!hbmpNew) {
			return NULL;
		}

		//shbmpNew==NULL̏ꍇz肵A
		//Ƃ̂݃LbVXV
		if (m_hbmp) {
			DeleteBitmap(m_hbmp);
		}

		m_hbmp		= hbmpNew;
		dibSize.cx	= width;
		dibSize.cy	= height;
		//CreateDIBSection͑y[WEZOgE
		m_lpPixels	= *lplpPixels;
		return m_hbmp;
	}

	*lplpPixels = m_lpPixels;
	return m_hbmp;
}

void CBitmapCache::FillSolidRect(COLORREF rgb, const RECT* lprc)
{
	LPBYTE lpPixels = m_lpPixels;
	const DWORD dwBmpBytes		= m_dibSize.cx * m_dibSize.cy;
	rgb = RGB2DIB(rgb);

	//TODO: MMX or SSE
	__asm {
		mov edi, dword ptr [lpPixels]
		mov ecx, dword ptr [dwBmpBytes]
		mov eax, dword ptr [rgb]
		cld
		rep stosd
	}
//	DWORD* p = (DWORD*)m_lpPixels;
//	DWORD* const pend = p + dwBmpBytes;
//	while (p < pend) {
//		*p++ = rgb;
//	}
}

COLORREF CBitmapCache::_GetPixel(int X, int Y) const
{
	if ((unsigned)X >= (unsigned)m_dibSize.cx || (unsigned)Y >= (unsigned)m_dibSize.cy) {
		return CLR_INVALID;
	}
	DWORD* lpPixels = (DWORD*)m_lpPixels;
	DWORD dib = lpPixels[Y * m_dibSize.cx + X];
	return dib;
}

void CBitmapCache::_SetPixelV(int X, int Y, DWORD bgr)
{
	if ((unsigned)X >= (unsigned)m_dibSize.cx || (unsigned)Y >= (unsigned)m_dibSize.cy) {
		return;
	}
	DWORD* lpPixels = (DWORD*)m_lpPixels;
	lpPixels[Y * m_dibSize.cx + X] = bgr;
}

void CBitmapCache::DrawHorizontalLine(int X1, int Y1, int X2, COLORREF rgb, int width)
{
	if (!m_dibSize.cx || !m_dibSize.cy) {
		return;
	}

	if (X1 > X2) {
		const int xx = X1;
		X1 = X2;
		X2 = xx;
	}

	const int xSize = m_dibSize.cx;
	const int ySize = m_dibSize.cy;
	X1 = Bound(X1, 0, xSize);
	X2 = Bound(X2, 0, xSize);
	Y1 = Bound(Y1, 0, ySize);
	width = Max(width, 1);
	const int Y2 = Bound(Y1 + width, 0, ySize);

	rgb = RGB2DIB(rgb);

	DWORD* lpPixels = (DWORD*)m_lpPixels + (Y1 * xSize + X1);
	const int Xd = X2 - X1;
	const int Yd = Y2 - Y1;
/*	for (int yy=Y1; yy<Y2; yy++) {
		for (int xx=X1; xx<X2; xx++) {
			_SetPixelV(xx, yy, rgb);
		}
	}

	for (int yy=Y1; yy<Y2; yy++, lpPixels += xSize) {
		__asm {
			mov edi, dword ptr [lpPixels]
			mov ecx, dword ptr [Xd]
			mov eax, dword ptr [rgb]
			cld
			rep stosd
		}
	}*/

	//ӖɃAZu
	__asm {
		mov ebx, dword ptr [Yd]
		mov edx, dword ptr [lpPixels]
		mov esi, dword ptr [xSize]
		cld
L1:
		mov edi, edx
		mov ecx, dword ptr [Xd]
		mov eax, dword ptr [rgb]
		rep stosd
		lea edx, dword ptr [edx+esi*4]
		dec ebx
		jnz L1
	}
}
