#include "settings.h"
#include <math.h>	//pow

static const TCHAR c_szGeneral[]  = _T("General");
static const TCHAR c_szFreeType[] = _T("FreeType");

bool CGdiPPSettings::LoadSettings()
{
	CCriticalSectionLock __lock;
	Assert(m_szFileName[0]);
	return LoadAppSettings(m_szFileName);
}

bool CGdiPPSettings::LoadSettings(HINSTANCE hModule)
{
	CCriticalSectionLock __lock;
	if (!::GetModuleFileName(hModule, m_szFileName, MAX_PATH - 8)) {
		return false;
	}

	_tcscpy(PathFindExtension(m_szFileName), _T(".ini"));
	return LoadAppSettings(m_szFileName);
}

int CGdiPPSettings::_GetFreeTypeProfileInt(LPCTSTR lpszKey, int nDefault, LPCTSTR lpszFile)
{
	const int retA = GetPrivateProfileInt(c_szFreeType, lpszKey, -1, lpszFile);
	const int retB = GetPrivateProfileInt(c_szFreeType, lpszKey, -2, lpszFile);

	if (retA == retB) {
		return retA;
	}
	return GetPrivateProfileInt(c_szGeneral, lpszKey, nDefault, lpszFile);
}

bool CGdiPPSettings::LoadAppSettings(LPCTSTR lpszFile)
{
	// eݒǂݍ
	// INIt@C̗:
	// [General]
	// HookChildProcesses=0
	// HintingMode=0
	// AntiAliasMode=0
	// EnableKerning=0
	// TextTuning=0
	// TextTuningR=0
	// TextTuningG=0
	// TextTuningB=0

	WritePrivateProfileString(NULL, NULL, NULL, lpszFile);

	m_nHintingMode = _GetFreeTypeProfileInt(_T("HintingMode"), 0, lpszFile);
	m_nHintingMode = Bound(m_nHintingMode, 0, 2);
	m_nAntiAliasMode = _GetFreeTypeProfileInt(_T("AntiAliasMode"), 0, lpszFile);
	m_nAntiAliasMode = Bound(m_nAntiAliasMode, 0, 3);
	m_bEnableKerning = !!_GetFreeTypeProfileInt(_T("EnableKerning"), false, lpszFile);
	m_bHookChildProcesses		= !!GetPrivateProfileInt(c_szGeneral, _T("HookChildProcesses"), false, lpszFile);

	const int nTextTuning	= _GetFreeTypeProfileInt(_T("TextTuning"),  0, lpszFile),
			  nTextTuningR	= _GetFreeTypeProfileInt(_T("TextTuningR"), 0, lpszFile),
			  nTextTuningG	= _GetFreeTypeProfileInt(_T("TextTuningG"), 0, lpszFile),
			  nTextTuningB	= _GetFreeTypeProfileInt(_T("TextTuningB"), 0, lpszFile);

	InitInitTuneTable();
	InitTuneTable(nTextTuning);
	InitTuneTableRGB(nTextTuningR, nTextTuningG, nTextTuningB);

	// OS̃o[WXPȍ~ǂ
	OSVERSIONINFO osvi = { sizeof(OSVERSIONINFO) };
	GetVersionEx(&osvi);
	m_bIsWinXPorLater = ((osvi.dwMajorVersion > 5) || ((osvi.dwMajorVersion == 5) && (osvi.dwMinorVersion >= 1)));

	m_arrExcludeFont.RemoveAll();
	m_arrExcludeModule.RemoveAll();

	// tHgw
	GetPrivateProfileString(L"General",L"ForceChangeFont",0, m_ForceFont, 32, lpszFile);

	// [Exclude]ZNV珜OtHgXgǂݍ
	AddListFromSection(_T("Exclude"), lpszFile, m_arrExcludeFont);
	// [ExcludeModule]ZNV珜OW[Xgǂݍ
	AddListFromSection(_T("ExcludeModule"), lpszFile, m_arrExcludeModule);
	return true;
}

template <class T>
bool CGdiPPSettings::AddListFromSection(LPCTSTR lpszSection, LPCTSTR lpszFile, CArray<T>& arr)
{
	LPTSTR  buffer = _GetPrivateProfileSection(lpszSection, lpszFile);
	if (buffer == NULL) {
		SetLastError(ERROR_NOT_ENOUGH_MEMORY);
		return false;
	}

	LPTSTR p = buffer;
	while (*p) {
		bool b = false;
		T str(p);
		for (int i = 0 ; i < arr.GetSize(); i++) {
			if (arr[i] == str) {
				b = true;
				break;
			}
		}
		if (!b) {
			arr.Add(str);
		}

		for (; *p; p++);
		p++;
	}
	delete[] buffer;
	return false;
}

LPTSTR CGdiPPSettings::_GetPrivateProfileSection(LPCTSTR lpszSection, LPCTSTR lpszFile)
{
	LPTSTR buffer = NULL;
	int nRes = 0;
	for (int cch = 256; ; cch += 256) {
		buffer = new TCHAR[cch];
		if (!buffer) {
			break;
		}

		ZeroMemory(buffer, sizeof(TCHAR) * cch);
		nRes = GetPrivateProfileSection(lpszSection, buffer, cch, lpszFile);
		if (nRes < cch - 1) {
			break;
		}
		delete[] buffer;
		buffer = NULL;
	}
	return buffer;
}

bool CGdiPPSettings::IsFontExcluded(LPCSTR lpFaceName)
{
	LPWSTR lpUnicode = _StrDupAtoW(lpFaceName);
	if (!lpUnicode) {
		return false;
	}

	bool b = IsFontExcluded(lpUnicode);
	delete[] lpUnicode;
	return b;
}

bool CGdiPPSettings::IsFontExcluded(LPCWSTR lpFaceName)
{
	StringHashFont* p = m_arrExcludeFont.Begin();
	StringHashFont* end = m_arrExcludeFont.End();
	StringHashFont str(lpFaceName);

	for(; p != end; ++p) {
		if (*p == str) {
			return true;
		}
	}
	return false;
}

bool CGdiPPSettings::IsProcessExcluded()
{
	StringHashModule* p = m_arrExcludeModule.Begin();
	StringHashModule* end = m_arrExcludeModule.End();
	for(; p != end; ++p) {
		if (GetModuleHandleW(p->c_str())) {
			return true;
		}
	}
	return false;
}

void CGdiPPSettings::InitInitTuneTable()
{
	int i, *table;
#define init_table(name) \
		for (i=0,table=name; i<sizeof(name)/sizeof(name[0]); i++) table[i] = i
	init_table(m_nTuneTable);
	init_table(m_nTuneTableR);
	init_table(m_nTuneTableG);
	init_table(m_nTuneTableB);
#undef init_table
}

// e[u֐ 0 - 12܂
void CGdiPPSettings::InitTuneTable(int v)
{
	int i;
	int col;
	double tmp, p;
	int* TuneTable = m_nTuneTable;

	if(v < 0 || v > 12)
		return;
	p = 1 - (v / (v + 10.0));
	for(i = 0;i < 256;i++){
	    tmp = i / 255.0;
        tmp = pow(tmp, p);
	    col = 255 - (int)(tmp * 255.0 + 0.5);
		TuneTable[255 - i] = col;
	}
}

// LCDpe[u֐ e0 - 12܂
void CGdiPPSettings::InitTuneTableRGB(int rv, int gv, int bv)
{
	int i;
	int col;
	double tmp, p;
	int* TuneTableR = m_nTuneTableR;
	int* TuneTableG = m_nTuneTableG;
	int* TuneTableB = m_nTuneTableB;

	if( rv < 0 || rv > 12 ||
	    gv < 0 || gv > 12 ||
	    bv < 0 || bv > 12)
		return;
	p = 1 - (rv / (rv + 10.0));
	for(i = 0;i < 256;i++){
	    tmp = i / 255.0;
        tmp = pow(tmp, p);
	    col = 255 - (int)(tmp * 255.0 + 0.5);
		TuneTableR[255 - i] = col;
	}
	p = 1 - (gv / (gv + 10.0));
	for(i = 0;i < 256;i++){
	    tmp = i / 255.0;
        tmp = pow(tmp, p);
	    col = 255 - (int)(tmp * 255.0 + 0.5);
		TuneTableG[255 - i] = col;
	}
	p = 1 - (bv / (bv + 10.0));
	for(i = 0;i < 256;i++){
	    tmp = i / 255.0;
        tmp = pow(tmp, p);
	    col = 255 - (int)(tmp * 255.0 + 0.5);
		TuneTableB[255 - i] = col;
	}
}

bool CGdiPPSettings::FT_AddFont(LPCTSTR lpFaceName)
{
	CCriticalSectionLock __lock;

	if(lpFaceName == NULL || _tcslen(lpFaceName) == 0)
		return false;

	if (FT_FontExists(lpFaceName)) {
		return false;
	}

	FontList fl(m_arrFontList.GetSize() + 1, lpFaceName);
	return !!m_arrFontList.Add(fl);
}

int CGdiPPSettings::FT_GetFontIdByName(LPCTSTR lpFaceName) const
{
	CCriticalSectionLock __lock;

	FontList* p = m_arrFontList.Begin();
	FontList* end = m_arrFontList.End();
	StringHashFont hash(lpFaceName);

	for(; p != end; ++p) {
		if (p->hash == hash) {
			return p->id;
		}
	}
	return 0;
}

LPCTSTR CGdiPPSettings::FT_GetFontNameById(int faceid) const
{
	CCriticalSectionLock __lock;

	FontList* p = m_arrFontList.Begin();
	FontList* end = m_arrFontList.End();
	for(; p != end; ++p) {
		if (p->id == faceid) {
			return p->hash.c_str();
		}
	}
	return NULL;
}
