#pragma once

#include "common.h"
#include "cache.h"

#ifndef PathFindExtension
LPTSTR PathFindExtension(LPCTSTR pszPath);
#endif

class CGdiPPSettings
{
private:
	int  m_nHintingMode;
	int  m_nAntiAliasMode;
	bool m_bIsWinXPorLater			: 1;
	bool m_bHookChildProcesses		: 1;
	bool m_bEnableKerning           : 1;
	bool							: 0;

	// A`GCAXpe[u
	int  m_nTuneTable[256];
	// LCDp
	int  m_nTuneTableR[256];
	int  m_nTuneTableG[256];
	int  m_nTuneTableB[256];

	// wtHg
	WCHAR m_ForceFont[32];

	// tHgFaceID(intgƂɂ)
	struct FontList {
		int id;
		StringHashFont hash;

		FontList() : id(0) {}
		FontList(int n, LPCTSTR name) : id(n), hash(name) {}
	};

	typedef CArray<StringHashFont>		FontHashArray;
	typedef CArray<StringHashModule>	ModuleHashArray;
	typedef CArray<FontList>			FontListArray;
	FontHashArray	m_arrExcludeFont;
	ModuleHashArray	m_arrExcludeModule;
	FontListArray	m_arrFontList;

	TCHAR m_szFileName[MAX_PATH];

	//INI̓ǂݍݏ
	bool LoadAppSettings(LPCTSTR lpszFile);
	static LPTSTR _GetPrivateProfileSection(LPCTSTR lpszSection, LPCTSTR lpszFile);
	static int _GetFreeTypeProfileInt(LPCTSTR lpszKey, int nDefault, LPCTSTR lpszFile);

	template <class T>
	static bool AddListFromSection(LPCTSTR lpszSection, LPCTSTR lpszFile, CArray<T>& arr);

	void InitInitTuneTable();
	void InitTuneTable(int v);
	void InitTuneTableRGB(int rv, int gv, int bv);

public:
	CGdiPPSettings()
		: m_nHintingMode(0)
		, m_nAntiAliasMode(0)
		, m_bIsWinXPorLater(false)
		, m_bHookChildProcesses(false)
		, m_bEnableKerning(false)
	{
		ZeroMemory(m_nTuneTable,  sizeof(m_nTuneTable));
		ZeroMemory(m_nTuneTableR, sizeof(m_nTuneTableR));
		ZeroMemory(m_nTuneTableG, sizeof(m_nTuneTableG));
		ZeroMemory(m_nTuneTableB, sizeof(m_nTuneTableB));
	}

	~CGdiPPSettings()
	{
	}

	// tHg݂Ƃ
	WCHAR* GetForceFontName() { return m_ForceFont; }

	int HintingMode() const { return m_nHintingMode; }
	int AntiAliasMode() const { return m_nAntiAliasMode; }
	bool IsWinXPorLater() const { return m_bIsWinXPorLater; }
	bool HookChildProcesses() const { return m_bHookChildProcesses; }
	bool EnableKerning() const { return m_bEnableKerning; }
	const int* GetTuneTable() const { return m_nTuneTable; }
	const int* GetTuneTableR() const { return m_nTuneTableR; }
	const int* GetTuneTableG() const { return m_nTuneTableG; }
	const int* GetTuneTableB() const { return m_nTuneTableB; }

	bool LoadSettings();
	bool LoadSettings(HINSTANCE hModule);

	bool IsFontExcluded(LPCSTR lpFaceName);
	bool IsFontExcluded(LPCWSTR lpFaceName);

	bool IsProcessExcluded();

	bool FT_AddFont(LPCTSTR lpFaceName);
	int  FT_GetFontIdByName(LPCTSTR lpFaceName) const;
	LPCTSTR FT_GetFontNameById(int faceid) const;
	bool FT_FontExists(LPCTSTR lpFaceName) const
	{
		return !!FT_GetFontIdByName(lpFaceName);
	}
};

extern CGdiPPSettings* g_pSettings;
